/*
 * VServer Context Quota
 * basic tools for vroot setup
 *
 * vrsetup.c
 *
 * (C) 2002-2003 Herbert Ptzl <herbert@13thfloor.at>
 *
 * Changelog:
 *
 *   0.10  - first public release
 *   0.12  - set/unset vroot device
 *   0.13  - fixed help bug
 *	   - cleanup for release
 *
 * This file is part of the vquota-tools.
 *
 * vquota-tools is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * vquota-tools is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with vquota-tools; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#define	VERSION	"V0.13"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h> 
#include <string.h> 

#include <sys/types.h>
#include <sys/ioctl.h>

#include "vroot.h"

extern 	int 		errno;
static  char *  	cmd_name;

static  char *  	cmd_vroot;
static  char *  	cmd_device;

static	char		opt_delete = 0;

static	char		opt_inc_use = 0;
static	char		opt_dec_use = 0;


#define	OPTIONS "hdID"


int	main(int argc, char *argv[])
{
        extern int optind;
        extern char *optarg;
        char c, errflg = 0;
	int fd, dfd;

        cmd_name = argv[0];
        while ((c = getopt(argc, argv, OPTIONS)) != EOF) {
            switch (c) {
            case 'h':
                fprintf(stderr,
                    "This is %s " VERSION "\n"
                    "options are:\n"
                    "-h        print this help message\n"
                    "-d        delete (unset) vroot device\n"
                    ,cmd_name);
                exit(0);
                break;
            case 'd':
                opt_delete = 1;
		break;
            case 'I':
                opt_inc_use = 1;
		break;
            case 'D':
                opt_dec_use = 1;
		break;
            case '?':
            default:
                errflg++;
                break;
            }
        }

	if (optind < argc) {
	    cmd_vroot = argv[optind++];
	} else {
	    errflg++;
	}
	if (optind < argc) {
	    cmd_device = argv[optind++];
	} else {
	    if (!opt_delete && !(opt_inc_use || opt_dec_use)) 
	    	errflg++;
	}
	
        if (errflg) {
            fprintf(stderr, 
                "Usage: %s -[" OPTIONS "] <vroot-dev> <real-root-dev>\n"
                "%s -h for help.\n",
                cmd_name, cmd_name);
            exit(1);
        }

	if ((fd = open (cmd_vroot, O_RDONLY)) < 0) {
	    int errsv = errno;
	
	    fprintf(stderr, 
	    	"%s: can't open device %s: %s\n",
		cmd_name, cmd_vroot, strerror (errsv));
	    exit(2);
	}
	
	if (opt_inc_use) {
	    ioctl (fd, VROOT_INC_USE, 0);
	    exit(-1);
	} 
	else if (opt_dec_use) {
	    ioctl (fd, VROOT_DEC_USE, 0);
	    exit(-1);
	} 
	else if (opt_delete) {
	    if (ioctl (fd, VROOT_CLR_DEV, 0) < 0) {
		perror ("ioctl: VROOT_CLR_DEV");
		exit(3);
	    }
	}
	else {	/* set vroot device */	
	    if ((dfd = open (cmd_device, O_RDONLY)) < 0) {
	    	int errsv = errno;
	
	    	fprintf(stderr,
	    	    "%s: can't open device %s: %s\n",
	    	    cmd_name, cmd_device, strerror (errsv));
		exit(4);
	    }
	    if (ioctl (fd, VROOT_SET_DEV, dfd) < 0) {
		perror ("ioctl: VROOT_SET_DEV");
		exit(3);
	    }
	    close(dfd);
	}
	close(fd);
 	exit(0);             	      
}
